import { useMessageBridge } from '@/api/message-bridge';
import { reactive, ref } from 'vue';
import { pinkLog } from '../setting/util';
import { ElLoading, ElMessage } from 'element-plus';
import { getPlatform, type OpenMcpSupportPlatform } from '@/api/platform';
import { getTour, loadSetting } from '@/hook/setting';
import { loadPanels } from '@/hook/panel';
import type { ConnectionType } from './connection-item';

export const connectionMethods = reactive<{
    current: ConnectionType,
    data: {
        value: ConnectionType,
        label: string
    }[]
}>({
    current: 'STDIO',
    data: [
        {
            value: 'STDIO',
            label: 'STDIO'
        },
        {
            value: 'SSE',
            label: 'SSE'
        },
        {
            value: 'STREAMABLE_HTTP',
            label: 'STREAMABLE_HTTP'
        }
    ]
});

export const connectionSettingRef = ref<any>(null);
export const connectionLogRef = ref<any>(null);

// 主 mcp 服务器的连接参数
export const connectionArgs = reactive({
    commandString: '',
    cwd: '',
    oauth: '',
    urlString: ''
});

export interface EnvItem {
    key: string
    value: string
}

export interface IConnectionEnv {
    data: EnvItem[]
    newKey: string
    newValue: string
}

export interface ConnectionResult {
    status: string
    clientId: string
    name: string
    version: string
}

export const connectionEnv = reactive<IConnectionEnv>({
    data: [],
    newKey: '',
    newValue: ''
});

export function makeEnv() {
    const env = {} as Record<string, string>;
    connectionEnv.data.forEach(item => {
        env[item.key] = item.value;
    });
    return env;
}


// 定义命令行参数接口
export interface McpOptions {
    connectionType: ConnectionType;
    // STDIO 特定选项
    command?: string;
    args?: string[];
    cwd?: string;
    env?: Record<string, string>;
    // SSE 特定选项
    url?: string;
    // 通用客户端选项
    clientName?: string;
    clientVersion?: string;
}

/**
 * @description 试图启动 mcp 服务器，它会
 * 1. 请求启动参数
 * 2. 启动 mcp 服务器
 * 3. 将本次的启动参数同步到本地
 * @param option 
 * @returns 
 */
export async function doConnect(
    option: {
        namespace: OpenMcpSupportPlatform
        updateCommandString?: boolean
    }
) {
    const {
        // updateCommandString 为 true 代表是初始化阶段
        namespace,
        updateCommandString = true
    } = option;

    //  如果是初始化，则需要请求启动参数
    if (updateCommandString) {
        pinkLog('请求启动参数');
        const connectionItem = await getLaunchSignature(namespace + '/launch-signature');
        connectionMethods.current = connectionItem.type;
        connectionArgs.commandString = connectionItem.commandString || '';
        connectionArgs.cwd = connectionItem.cwd || '';
        connectionArgs.oauth = connectionItem.oauth || '';
        connectionArgs.urlString = connectionItem.url || '';
    }

    if (connectionMethods.current === 'STDIO') {
        return await launchStdio(namespace);
    } else {
        return await launchRemote(namespace);
    }
}

async function launchStdio(namespace: string) {
    const bridge = useMessageBridge();
    const env = makeEnv();

    const commandComponents = connectionArgs.commandString.split(/\s+/g);
    const command = commandComponents[0];
    commandComponents.shift();

    const connectOption = {
        connectionType: 'STDIO',
        command: command,
        args: commandComponents,
        cwd: connectionArgs.cwd,
        clientName: 'openmcp.connect.STDIO',
        clientVersion: '0.0.1',
        env
    };

    const { code, msg } = await bridge.commandRequest<ConnectionResult>('connect', connectOption);

    connectionResult.success = (code === 200);

    if (code === 200) {

        const message = `connect to ${msg.name} ${msg.version} success, clientId: ${msg.clientId}`;
        connectionResult.logString.push({ type: 'info', message });        

        connectionResult.serverInfo.name = msg.name || '';
        connectionResult.serverInfo.version = msg.version || '';
        connectionResult.clientId = msg.clientId || '';

        // 同步信息到 后端
        const commandComponents = connectionArgs.commandString.split(/\s+/g);
        const command = commandComponents[0];
        commandComponents.shift();

        const clientStdioConnectionItem = {
            serverInfo: connectionResult.serverInfo,
            connectionType: 'STDIO',
            name: 'openmcp.connect.STDIO',
            command: command,
            args: commandComponents,
            cwd: connectionArgs.cwd,
            env
        };

        bridge.postMessage({
            command: namespace + '/update-connection-sigature',
            data: JSON.parse(JSON.stringify(clientStdioConnectionItem))
        });

    } else {
        const messaage = msg.toString();
        connectionResult.logString.push({
            type: 'error',
            message: messaage
        });

        ElMessage.error(messaage);
    }
}

async function launchRemote(namespace: string) {
    const bridge = useMessageBridge();
    const env = makeEnv();

    const connectOption: McpOptions = {
        connectionType: connectionMethods.current,
        url: connectionArgs.urlString,
        clientName: 'openmcp.connect.' + connectionMethods.current,
        clientVersion: '0.0.1',
        env
    };

    const { code, msg } = await bridge.commandRequest<ConnectionResult>('connect', connectOption);

    connectionResult.success = (code === 200);

    if (code === 200) {
        const message = `connect to ${msg.name} ${msg.version} success, clientId: ${msg.clientId}`;

        connectionResult.logString.push({
            type: 'info',
            message: message
        });

        connectionResult.serverInfo.name = msg.name || '';
        connectionResult.serverInfo.version = msg.version || '';
        connectionResult.clientId = msg.clientId || '';
        
        // 同步信息到 vscode
        const clientSseConnectionItem = {
            serverInfo: connectionResult.serverInfo,
            connectionType: connectionMethods.current,
            name: 'openmcp.connect.' + connectionMethods.current,
            url: connectionArgs.urlString,
            oauth: connectionArgs.oauth,
            env: env
        };

        bridge.postMessage({
            command: namespace + '/update-connection-sigature',
            data: JSON.parse(JSON.stringify(clientSseConnectionItem))
        });

    } else {
        const message = msg.toString();
        connectionResult.logString.push({
            type: 'error',
            message: message
        });

        ElMessage.error(message);
    }
}


async function getLaunchSignature(signatureName: string) {
    const bridge = useMessageBridge();
    const { code, msg } = await bridge.commandRequest(signatureName);

    return msg;
}

export function doReconnect() {
    // TODO: finish this
    console.log();
}

export const connectionResult = reactive<{
    success: boolean,
    logString: { type: 'info' | 'error' | 'warning', message: string }[],
    serverInfo: {
        name: string,
        version: string
    },
    clientId: string
}>({
    success: false,
    logString: [],
    serverInfo: {
        name: '',
        version: ''
    },
    clientId: ''
});



export const envVarStatus = {
    launched: false
};

function lookupEnvVar(varNames: string[]) {
    const bridge = useMessageBridge();

	return new Promise<string[] | undefined>((resolve, reject) => {
		bridge.addCommandListener('lookup-env-var', data => {
			const { code, msg } = data;
			
			if (code === 200) {
				connectionResult.logString.push({
					type: 'info',
					message: '预设环境变量同步完成'
				});

				resolve(msg);
			} else {
				connectionResult.logString.push({
					type: 'error',
					message: '预设环境变量同步失败: ' + msg
				});

				resolve(undefined);
			}
		}, { once: true });

		console.log(varNames);
		
		
		bridge.postMessage({
			command: 'lookup-env-var',
			data: {
				keys: varNames
			}
		})
	});
}


export async function handleEnvSwitch(enabled: boolean) {
	const presetVars = ['HOME', 'LOGNAME', 'PATH', 'SHELL', 'TERM', 'USER'];

	if (enabled) {
		const values = await lookupEnvVar(presetVars);

		if (values) {
			// 将 key values 合并进 connectionEnv.data 中
			// 若已有相同的 key, 则替换 value
			for (let i = 0; i < presetVars.length; i++) {
				const key = presetVars[i];
				const value = values[i];
				const sameNameItems = connectionEnv.data.filter(item => item.key === key);
				if (sameNameItems.length > 0) {
					const conflictItem = sameNameItems[0];
					conflictItem.value = value;
				} else {
					connectionEnv.data.push({
						key: key, value: value
					});
				}
			}
		}
	} else {
		// 清空 connectionEnv.data 中所有 key 为 presetVars 的项
		const reserveItems = connectionEnv.data.filter(item => !presetVars.includes(item.key));
		connectionEnv.data = reserveItems;
	}
}

export async function loadEnvVar() {
    return await handleEnvSwitch(true);
}

export async function initialise() {

	pinkLog('准备请求设置');
    
    const loading = ElLoading.service({
		fullscreen: true,
		lock: true,
		text: 'Loading',
		background: 'rgba(0, 0, 0, 0.7)'
	});
    const platform = getPlatform();
    
	// 加载全局设置
	loadSetting();

	// 设置环境变量
	await loadEnvVar();

	// 获取引导状态
	await getTour();

	// 尝试进行初始化连接
	await doConnect({
		namespace: platform,
		updateCommandString: true
	});

	// loading panels
	await loadPanels();

	loading.close();
}